@@ -3,10 +3,7 @@
 You can specify your local database settings in LOCAL_PyGreSQL.py.
 """
 
-try:
-    import unittest2 as unittest  # for Python < 2.7
-except ImportError:
-    import unittest
+import unittest
 
 if not (hasattr(unittest, 'skip')
         and hasattr(unittest.TestCase, 'setUpClass')
@@ -18,4 +15,4 @@ if not (hasattr(unittest, 'skip')
 def discover():
     loader = unittest.TestLoader()
     suite = loader.discover('.')
-    return suite
\ No newline at end of file
+    return suite
