@@ -1,33 +1,26 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
-
-from __future__ import print_function
 
 import unittest
-
+from contextlib import suppress
 from functools import partial
-from time import sleep
 from threading import Thread
+from time import sleep
 
-from pg import *
+from pg import (
+    DB,
+    DatabaseError,
+    Error,
+    IntegrityError,
+    NotificationHandler,
+    NotSupportedError,
+    ProgrammingError,
+)
 
-# We need a database to test against.  If LOCAL_PyGreSQL.py exists we will
-# get our information from that.  Otherwise we use the defaults.
-dbname = 'unittest'
-dbhost = None
-dbport = 5432
-
-try:
-    from .LOCAL_PyGreSQL import *  # noqa: F401
-except (ImportError, ValueError):
-    try:
-        from LOCAL_PyGreSQL import *  # noqa: F401
-    except ImportError:
-        pass
+from .config import dbhost, dbname, dbpasswd, dbport, dbuser
 
 
 def open_db():
-    db = DB(dbname, dbhost, dbport)
+    db = DB(dbname, dbhost, dbport, user=dbuser, passwd=dbpasswd)
     db.query("SET DATESTYLE TO 'ISO'")
     db.query("SET TIME ZONE 'EST5EDT'")
     db.query("SET DEFAULT_WITH_OIDS=FALSE")
@@ -40,48 +33,40 @@ class UtilityTest(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        """Recreate test tables and schemas"""
+        """Recreate test tables and schemas."""
         db = open_db()
-        try:
+        with suppress(Exception):
             db.query("DROP VIEW _test_vschema")
-        except Exception:
-            pass
-        try:
+        with suppress(Exception):
             db.query("DROP TABLE _test_schema")
-        except Exception:
-            pass
         db.query("CREATE TABLE _test_schema"
                  " (_test int PRIMARY KEY, _i interval, dvar int DEFAULT 999)")
         db.query("CREATE VIEW _test_vschema AS"
                  " SELECT _test, 'abc'::text AS _test2 FROM _test_schema")
         for t in ('_test1', '_test2'):
-            try:
+            with suppress(Exception):
                 db.query("CREATE SCHEMA " + t)
-            except Exception:
-                pass
-            try:
-                db.query("DROP TABLE %s._test_schema" % (t,))
-            except Exception:
-                pass
-            db.query("CREATE TABLE %s._test_schema"
-                     " (%s int PRIMARY KEY)" % (t, t))
+            with suppress(Exception):
+                db.query(f"DROP TABLE {t}._test_schema")
+            db.query(f"CREATE TABLE {t}._test_schema"
+                     f" ({t} int PRIMARY KEY)")
         db.close()
 
     def setUp(self):
-        """Setup test tables or empty them if they already exist."""
+        """Set up test tables or empty them if they already exist."""
         db = open_db()
         db.query("TRUNCATE TABLE _test_schema")
         for t in ('_test1', '_test2'):
-            db.query("TRUNCATE TABLE %s._test_schema" % t)
+            db.query(f"TRUNCATE TABLE {t}._test_schema")
         db.close()
 
     def test_invalid_name(self):
-        """Make sure that invalid table names are caught"""
+        """Make sure that invalid table names are caught."""
         db = open_db()
         self.assertRaises(NotSupportedError, db.get_attnames, 'x.y.z')
 
     def test_schema(self):
-        """Does it differentiate the same table name in different schemas"""
+        """Check differentiation of same table name in different schemas."""
         db = open_db()
         # see if they differentiate the table names properly
         self.assertEqual(
@@ -163,7 +148,7 @@ class UtilityTest(unittest.TestCase):
         try:
             db.query("INSERT INTO _test_schema VALUES (1234)")
         except DatabaseError as error:
-            self.assertTrue(isinstance(error, IntegrityError))
+            self.assertIsInstance(error, IntegrityError)
             # the SQLSTATE error code for unique violation is 23505
             # noinspection PyUnresolvedReferences
             self.assertEqual(error.sqlstate, '23505')
@@ -223,7 +208,7 @@ class UtilityTest(unittest.TestCase):
         thread.start()
         try:
             # Wait until the thread has started.
-            for n in range(500):
+            for _n in range(500):
                 if target.listening:
                     break
                 sleep(0.01)
@@ -245,7 +230,7 @@ class UtilityTest(unittest.TestCase):
             if two_payloads:
                 db2.commit()
             # Wait until the notification has been caught.
-            for n in range(500):
+            for _n in range(500):
                 if arg_dict['called'] or self.notify_timeout:
                     break
                 sleep(0.01)
@@ -253,7 +238,7 @@ class UtilityTest(unittest.TestCase):
             self.assertTrue(arg_dict['called'])
             self.assertEqual(arg_dict['event'], 'event_1')
             self.assertEqual(arg_dict['extra'], 'payload 1')
-            self.assertTrue(isinstance(arg_dict['pid'], int))
+            self.assertIsInstance(arg_dict['pid'], int)
             self.assertFalse(self.notify_timeout)
             arg_dict['called'] = False
             self.assertTrue(thread.is_alive())
@@ -264,7 +249,7 @@ class UtilityTest(unittest.TestCase):
                 db2.query("notify stop_event_1, 'payload 2'")
             db2.close()
             # Wait until the notification has been caught.
-            for n in range(500):
+            for _n in range(500):
                 if arg_dict['called'] or self.notify_timeout:
                     break
                 sleep(0.01)
@@ -272,7 +257,7 @@ class UtilityTest(unittest.TestCase):
             self.assertTrue(arg_dict['called'])
             self.assertEqual(arg_dict['event'], 'stop_event_1')
             self.assertEqual(arg_dict['extra'], 'payload 2')
-            self.assertTrue(isinstance(arg_dict['pid'], int))
+            self.assertIsInstance(arg_dict['pid'], int)
             self.assertFalse(self.notify_timeout)
             thread.join(5)
             self.assertFalse(thread.is_alive())
