@@ -1,5 +1,4 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
 
 """Test the classic PyGreSQL interface.
 
@@ -10,82 +9,82 @@ Contributed by Christoph Zwerschke.
 These tests do not need a database to test against.
 """
 
-import unittest
+from __future__ import annotations
 
 import json
 import re
+import unittest
+from datetime import timedelta
+from decimal import Decimal
+from typing import Any, Sequence
 
 import pg  # the module under test
 
-from datetime import timedelta
-
-try:  # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
-    long
-except NameError:  # Python >= 3.0
-    long = int
-
-try:  # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
-    unicode
-except NameError:  # Python >= 3.0
-    unicode = str
-
 
 class TestHasConnect(unittest.TestCase):
     """Test existence of basic pg module functions."""
 
-    def testhasPgError(self):
+    def test_has_pg_error(self):
         self.assertTrue(issubclass(pg.Error, Exception))
 
-    def testhasPgWarning(self):
+    def test_has_pg_warning(self):
         self.assertTrue(issubclass(pg.Warning, Exception))
 
-    def testhasPgInterfaceError(self):
+    def test_has_pg_interface_error(self):
         self.assertTrue(issubclass(pg.InterfaceError, pg.Error))
 
-    def testhasPgDatabaseError(self):
+    def test_has_pg_database_error(self):
         self.assertTrue(issubclass(pg.DatabaseError, pg.Error))
 
-    def testhasPgInternalError(self):
+    def test_has_pg_internal_error(self):
         self.assertTrue(issubclass(pg.InternalError, pg.DatabaseError))
 
-    def testhasPgOperationalError(self):
+    def test_has_pg_operational_error(self):
         self.assertTrue(issubclass(pg.OperationalError, pg.DatabaseError))
 
-    def testhasPgProgrammingError(self):
+    def test_has_pg_programming_error(self):
         self.assertTrue(issubclass(pg.ProgrammingError, pg.DatabaseError))
 
-    def testhasPgIntegrityError(self):
+    def test_has_pg_integrity_error(self):
         self.assertTrue(issubclass(pg.IntegrityError, pg.DatabaseError))
 
-    def testhasPgDataError(self):
+    def test_has_pg_data_error(self):
         self.assertTrue(issubclass(pg.DataError, pg.DatabaseError))
 
-    def testhasPgNotSupportedError(self):
+    def test_has_pg_not_supported_error(self):
         self.assertTrue(issubclass(pg.NotSupportedError, pg.DatabaseError))
 
-    def testhasPgInvalidResultError(self):
+    def test_has_pg_invalid_result_error(self):
         self.assertTrue(issubclass(pg.InvalidResultError, pg.DataError))
 
-    def testhasPgNoResultError(self):
+    def test_has_pg_no_result_error(self):
         self.assertTrue(issubclass(pg.NoResultError, pg.InvalidResultError))
 
-    def testhasPgMultipleResultsError(self):
+    def test_has_pg_multiple_results_error(self):
         self.assertTrue(
             issubclass(pg.MultipleResultsError, pg.InvalidResultError))
 
-    def testhasConnect(self):
+    def test_has_connection_type(self):
+        self.assertIsInstance(pg.Connection, type)
+        self.assertEqual(pg.Connection.__name__, 'Connection')
+
+    def test_has_query_type(self):
+        self.assertIsInstance(pg.Query, type)
+        self.assertEqual(pg.Query.__name__, 'Query')
+
+    def test_has_connect(self):
         self.assertTrue(callable(pg.connect))
 
-    def testhasEscapeString(self):
+    def test_has_escape_string(self):
         self.assertTrue(callable(pg.escape_string))
 
-    def testhasEscapeBytea(self):
+    def test_has_escape_bytea(self):
         self.assertTrue(callable(pg.escape_bytea))
 
-    def testhasUnescapeBytea(self):
+    def test_has_unescape_bytea(self):
         self.assertTrue(callable(pg.unescape_bytea))
 
-    def testDefHost(self):
+    def test_def_host(self):
         d0 = pg.get_defhost()
         d1 = 'pgtesthost'
         pg.set_defhost(d1)
@@ -93,7 +92,7 @@ class TestHasConnect(unittest.TestCase):
         pg.set_defhost(d0)
         self.assertEqual(pg.get_defhost(), d0)
 
-    def testDefPort(self):
+    def test_def_port(self):
         d0 = pg.get_defport()
         d1 = 1234
         pg.set_defport(d1)
@@ -105,7 +104,7 @@ class TestHasConnect(unittest.TestCase):
             d0 = None
         self.assertEqual(pg.get_defport(), d0)
 
-    def testDefOpt(self):
+    def test_def_opt(self):
         d0 = pg.get_defopt()
         d1 = '-h pgtesthost -p 1234'
         pg.set_defopt(d1)
@@ -113,7 +112,7 @@ class TestHasConnect(unittest.TestCase):
         pg.set_defopt(d0)
         self.assertEqual(pg.get_defopt(), d0)
 
-    def testDefBase(self):
+    def test_def_base(self):
         d0 = pg.get_defbase()
         d1 = 'pgtestdb'
         pg.set_defbase(d1)
@@ -121,18 +120,18 @@ class TestHasConnect(unittest.TestCase):
         pg.set_defbase(d0)
         self.assertEqual(pg.get_defbase(), d0)
 
-    def testPqlibVersion(self):
+    def test_pqlib_version(self):
         # noinspection PyUnresolvedReferences
         v = pg.get_pqlib_version()
-        self.assertIsInstance(v, long)
-        self.assertGreater(v, 90000)
-        self.assertLess(v, 160000)
+        self.assertIsInstance(v, int)
+        self.assertGreater(v, 100000)
+        self.assertLess(v, 170000)
 
 
 class TestParseArray(unittest.TestCase):
     """Test the array parser."""
 
-    test_strings = [
+    test_strings: Sequence[tuple[str, type | None, Any]] = [
         ('', str, ValueError),
         ('{}', None, []),
         ('{}', str, []),
@@ -229,7 +228,7 @@ class TestParseArray(unittest.TestCase):
         ('[3:5]={{1,2,3},{4,5,6}}', int, ValueError),
         ('[1:1][-2:-1][3:5]={{1,2,3},{4,5,6}}', int, ValueError)]
 
-    def testParserParams(self):
+    def test_parser_params(self):
         f = pg.cast_array
         self.assertRaises(TypeError, f)
         self.assertRaises(TypeError, f, None)
@@ -248,13 +247,13 @@ class TestParseArray(unittest.TestCase):
         self.assertEqual(f('{}', str), [])
         self.assertEqual(f('{}', str, b';'), [])
 
-    def testParserSimple(self):
+    def test_parser_simple(self):
         r = pg.cast_array('{a,b,c}')
         self.assertIsInstance(r, list)
         self.assertEqual(len(r), 3)
         self.assertEqual(r, ['a', 'b', 'c'])
 
-    def testParserNested(self):
+    def test_parser_nested(self):
         f = pg.cast_array
         r = f('{{a,b,c}}')
         self.assertIsInstance(r, list)
@@ -279,29 +278,29 @@ class TestParseArray(unittest.TestCase):
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0], 'b')
         r = f('{{{{{{{abc}}}}}}}')
-        for i in range(7):
+        for _i in range(7):
             self.assertIsInstance(r, list)
             self.assertEqual(len(r), 1)
             # noinspection PyUnresolvedReferences
             r = r[0]
         self.assertEqual(r, 'abc')
 
-    def testParserTooDeeplyNested(self):
+    def test_parser_too_deeply_nested(self):
         f = pg.cast_array
         for n in 3, 5, 9, 12, 16, 32, 64, 256:
-            r = '%sa,b,c%s' % ('{' * n, '}' * n)
+            s = '{' * n + 'a,b,c' + '}' * n
             if n > 16:  # hard coded maximum depth
-                self.assertRaises(ValueError, f, r)
+                self.assertRaises(ValueError, f, s)
             else:
-                r = f(r)
-                for i in range(n - 1):
+                r = f(s)
+                for _i in range(n - 1):
                     self.assertIsInstance(r, list)
                     self.assertEqual(len(r), 1)
                     r = r[0]
                 self.assertEqual(len(r), 3)
                 self.assertEqual(r, ['a', 'b', 'c'])
 
-    def testParserCast(self):
+    def test_parser_cast(self):
         f = pg.cast_array
         self.assertEqual(f('{1}'), ['1'])
         self.assertEqual(f('{1}', None), ['1'])
@@ -313,10 +312,10 @@ class TestParseArray(unittest.TestCase):
         self.assertEqual(f('{a}', str), ['a'])
 
         def cast(s):
-            return '%s is ok' % s
+            return f'{s} is ok'
         self.assertEqual(f('{a}', cast), ['a is ok'])
 
-    def testParserDelim(self):
+    def test_parser_delim(self):
         f = pg.cast_array
         self.assertEqual(f('{1,2}'), ['1', '2'])
         self.assertEqual(f('{1,2}', delim=b','), ['1', '2'])
@@ -324,7 +323,7 @@ class TestParseArray(unittest.TestCase):
         self.assertEqual(f('{1;2}', delim=b';'), ['1', '2'])
         self.assertEqual(f('{1,2}', delim=b';'), ['1,2'])
 
-    def testParserWithData(self):
+    def test_parser_with_data(self):
         f = pg.cast_array
         for string, cast, expected in self.test_strings:
             if expected is ValueError:
@@ -332,7 +331,7 @@ class TestParseArray(unittest.TestCase):
             else:
                 self.assertEqual(f(string, cast), expected)
 
-    def testParserWithoutCast(self):
+    def test_parser_without_cast(self):
         f = pg.cast_array
 
         for string, cast, expected in self.test_strings:
@@ -343,7 +342,7 @@ class TestParseArray(unittest.TestCase):
             else:
                 self.assertEqual(f(string), expected)
 
-    def testParserWithDifferentDelimiter(self):
+    def test_parser_with_different_delimiter(self):
         f = pg.cast_array
 
         def replace_comma(value):
@@ -366,7 +365,7 @@ class TestParseArray(unittest.TestCase):
 class TestParseRecord(unittest.TestCase):
     """Test the record parser."""
 
-    test_strings = [
+    test_strings: Sequence[tuple[str, type | tuple[type, ...] | None, Any]] = [
         ('', None, ValueError),
         ('', str, ValueError),
         ('(', None, ValueError),
@@ -504,7 +503,7 @@ class TestParseRecord(unittest.TestCase):
         ('(fuzzy dice,"42","1.9375")', (str, int, float),
             ('fuzzy dice', 42, 1.9375))]
 
-    def testParserParams(self):
+    def test_parser_params(self):
         f = pg.cast_record
         self.assertRaises(TypeError, f)
         self.assertRaises(TypeError, f, None)
@@ -523,27 +522,28 @@ class TestParseRecord(unittest.TestCase):
         self.assertEqual(f('()', str), (None,))
         self.assertEqual(f('()', str, b';'), (None,))
 
-    def testParserSimple(self):
+    def test_parser_simple(self):
         r = pg.cast_record('(a,b,c)')
         self.assertIsInstance(r, tuple)
         self.assertEqual(len(r), 3)
         self.assertEqual(r, ('a', 'b', 'c'))
 
-    def testParserNested(self):
+    def test_parser_nested(self):
         f = pg.cast_record
         self.assertRaises(ValueError, f, '((a,b,c))')
         self.assertRaises(ValueError, f, '((a,b),(c,d))')
         self.assertRaises(ValueError, f, '((a),(b),(c))')
         self.assertRaises(ValueError, f, '(((((((abc)))))))')
 
-    def testParserManyElements(self):
+    def test_parser_many_elements(self):
         f = pg.cast_record
         for n in 3, 5, 9, 12, 16, 32, 64, 256:
-            r = '(%s)' % ','.join(map(str, range(n)))
-            r = f(r, int)
+            s = ','.join(map(str, range(n)))
+            s = f'({s})'
+            r = f(s, int)
             self.assertEqual(r, tuple(range(n)))
 
-    def testParserCastUniform(self):
+    def test_parser_cast_uniform(self):
         f = pg.cast_record
         self.assertEqual(f('(1)'), ('1',))
         self.assertEqual(f('(1)', None), ('1',))
@@ -555,10 +555,10 @@ class TestParseRecord(unittest.TestCase):
         self.assertEqual(f('(a)', str), ('a',))
 
         def cast(s):
-            return '%s is ok' % s
+            return f'{s} is ok'
         self.assertEqual(f('(a)', cast), ('a is ok',))
 
-    def testParserCastNonUniform(self):
+    def test_parser_cast_non_uniform(self):
         f = pg.cast_record
         self.assertEqual(f('(1)', []), ('1',))
         self.assertEqual(f('(1)', [None]), ('1',))
@@ -582,11 +582,11 @@ class TestParseRecord(unittest.TestCase):
             (1, 'a', 2, 'b', 3, 'c'))
 
         def cast1(s):
-            return '%s is ok' % s
+            return f'{s} is ok'
         self.assertEqual(f('(a)', [cast1]), ('a is ok',))
 
         def cast2(s):
-            return 'and %s is ok, too' % s
+            return f'and {s} is ok, too'
         self.assertEqual(
             f('(a,b)', [cast1, cast2]), ('a is ok', 'and b is ok, too'))
         self.assertRaises(ValueError, f, '(a)', [cast1, cast2])
@@ -595,7 +595,7 @@ class TestParseRecord(unittest.TestCase):
             f('(1,2,3,4,5,6)', [int, float, str, None, cast1, cast2]),
             (1, 2.0, '3', '4', '5 is ok', 'and 6 is ok, too'))
 
-    def testParserDelim(self):
+    def test_parser_delim(self):
         f = pg.cast_record
         self.assertEqual(f('(1,2)'), ('1', '2'))
         self.assertEqual(f('(1,2)', delim=b','), ('1', '2'))
@@ -603,7 +603,7 @@ class TestParseRecord(unittest.TestCase):
         self.assertEqual(f('(1;2)', delim=b';'), ('1', '2'))
         self.assertEqual(f('(1,2)', delim=b';'), ('1,2',))
 
-    def testParserWithData(self):
+    def test_parser_with_data(self):
         f = pg.cast_record
         for string, cast, expected in self.test_strings:
             if expected is ValueError:
@@ -611,7 +611,7 @@ class TestParseRecord(unittest.TestCase):
             else:
                 self.assertEqual(f(string, cast), expected)
 
-    def testParserWithoutCast(self):
+    def test_parser_without_cast(self):
         f = pg.cast_record
 
         for string, cast, expected in self.test_strings:
@@ -622,7 +622,7 @@ class TestParseRecord(unittest.TestCase):
             else:
                 self.assertEqual(f(string), expected)
 
-    def testParserWithDifferentDelimiter(self):
+    def test_parser_with_different_delimiter(self):
         f = pg.cast_record
 
         def replace_comma(value):
@@ -646,7 +646,7 @@ class TestParseRecord(unittest.TestCase):
 class TestParseHStore(unittest.TestCase):
     """Test the hstore parser."""
 
-    test_strings = [
+    test_strings: Sequence[tuple[str, Any]] = [
         ('', {}),
         ('=>', ValueError),
         ('""=>', ValueError),
@@ -677,7 +677,7 @@ class TestParseHStore(unittest.TestCase):
         (r'k\=\>v=>"k=>v"', {'k=>v': 'k=>v'}),
         ('a\\,b=>a,b=>a', {'a,b': 'a', 'b': 'a'})]
 
-    def testParser(self):
+    def test_parser(self):
         f = pg.cast_hstore
 
         self.assertRaises(TypeError, f)
@@ -695,7 +695,7 @@ class TestParseHStore(unittest.TestCase):
 class TestCastInterval(unittest.TestCase):
     """Test the interval typecast function."""
 
-    intervals = [
+    intervals: Sequence[tuple[tuple[int, ...], tuple[str, ...]]] = [
         ((0, 0, 0, 1, 0, 0, 0),
             ('1:00:00', '01:00:00', '@ 1 hour', 'PT1H')),
         ((0, 0, 0, -1, 0, 0, 0),
@@ -854,16 +854,16 @@ class TestCastInterval(unittest.TestCase):
              '@ 10 mons 3 days -3 hours -55 mins -5.999993 secs ago',
              'P-10M-3DT3H55M5.999993S'))]
 
-    def testCastInterval(self):
+    def test_cast_interval(self):
+        from pg.cast import cast_interval
         for result, values in self.intervals:
-            f = pg.cast_interval
             years, mons, days, hours, mins, secs, usecs = result
             days += 365 * years + 30 * mons
             interval = timedelta(
                 days=days, hours=hours, minutes=mins,
                 seconds=secs, microseconds=usecs)
             for value in values:
-                self.assertEqual(f(value), interval)
+                self.assertEqual(cast_interval(value), interval)
 
 
 class TestEscapeFunctions(unittest.TestCase):
@@ -876,47 +876,47 @@ class TestEscapeFunctions(unittest.TestCase):
 
     """
 
-    def testEscapeString(self):
+    def test_escape_string(self):
         f = pg.escape_string
-        r = f(b'plain')
-        self.assertIsInstance(r, bytes)
-        self.assertEqual(r, b'plain')
-        r = f(u'plain')
-        self.assertIsInstance(r, unicode)
-        self.assertEqual(r, u'plain')
-        r = f("that's cheese")
-        self.assertIsInstance(r, str)
-        self.assertEqual(r, "that''s cheese")
+        b = f(b'plain')
+        self.assertIsInstance(b, bytes)
+        self.assertEqual(b, b'plain')
+        s = f('plain')
+        self.assertIsInstance(s, str)
+        self.assertEqual(s, 'plain')
+        s = f("that's cheese")
+        self.assertIsInstance(s, str)
+        self.assertEqual(s, "that''s cheese")
 
-    def testEscapeBytea(self):
+    def test_escape_bytea(self):
         f = pg.escape_bytea
-        r = f(b'plain')
-        self.assertIsInstance(r, bytes)
-        self.assertEqual(r, b'plain')
-        r = f(u'plain')
-        self.assertIsInstance(r, unicode)
-        self.assertEqual(r, u'plain')
-        r = f("that's cheese")
-        self.assertIsInstance(r, str)
-        self.assertEqual(r, "that''s cheese")
+        b = f(b'plain')
+        self.assertIsInstance(b, bytes)
+        self.assertEqual(b, b'plain')
+        s = f('plain')
+        self.assertIsInstance(s, str)
+        self.assertEqual(s, 'plain')
+        s = f("that's cheese")
+        self.assertIsInstance(s, str)
+        self.assertEqual(s, "that''s cheese")
 
-    def testUnescapeBytea(self):
+    def test_unescape_bytea(self):
         f = pg.unescape_bytea
         r = f(b'plain')
         self.assertIsInstance(r, bytes)
         self.assertEqual(r, b'plain')
-        r = f(u'plain')
+        r = f('plain')
         self.assertIsInstance(r, bytes)
         self.assertEqual(r, b'plain')
         r = f(b"das is' k\\303\\244se")
         self.assertIsInstance(r, bytes)
-        self.assertEqual(r, u"das is' käse".encode('utf-8'))
-        r = f(u"das is' k\\303\\244se")
+        self.assertEqual(r, "das is' käse".encode())
+        r = f("das is' k\\303\\244se")
         self.assertIsInstance(r, bytes)
-        self.assertEqual(r, u"das is' käse".encode('utf-8'))
+        self.assertEqual(r, "das is' käse".encode())
         r = f(b'O\\000ps\\377!')
         self.assertEqual(r, b'O\x00ps\xff!')
-        r = f(u'O\\000ps\\377!')
+        r = f('O\\000ps\\377!')
         self.assertEqual(r, b'O\x00ps\xff!')
 
 
@@ -928,10 +928,10 @@ class TestConfigFunctions(unittest.TestCase):
 
     """
 
-    def testGetDatestyle(self):
+    def test_get_datestyle(self):
         self.assertIsNone(pg.get_datestyle())
 
-    def testSetDatestyle(self):
+    def test_set_datestyle(self):
         datestyle = pg.get_datestyle()
         try:
             pg.set_datestyle('ISO, YMD')
@@ -951,12 +951,12 @@ class TestConfigFunctions(unittest.TestCase):
         finally:
             pg.set_datestyle(datestyle)
 
-    def testGetDecimalPoint(self):
+    def test_get_decimal_point(self):
         r = pg.get_decimal_point()
         self.assertIsInstance(r, str)
         self.assertEqual(r, '.')
 
-    def testSetDecimalPoint(self):
+    def test_set_decimal_point(self):
         point = pg.get_decimal_point()
         try:
             pg.set_decimal_point('*')
@@ -969,12 +969,12 @@ class TestConfigFunctions(unittest.TestCase):
         self.assertIsInstance(r, str)
         self.assertEqual(r, point)
 
-    def testGetDecimal(self):
+    def test_get_decimal(self):
         r = pg.get_decimal()
-        self.assertIs(r, pg.Decimal)
+        self.assertIs(r, Decimal)
 
-    def testSetDecimal(self):
-        decimal_class = pg.Decimal
+    def test_set_decimal(self):
+        decimal_class = Decimal
         try:
             pg.set_decimal(int)
             r = pg.get_decimal()
@@ -984,12 +984,12 @@ class TestConfigFunctions(unittest.TestCase):
         r = pg.get_decimal()
         self.assertIs(r, decimal_class)
 
-    def testGetBool(self):
+    def test_get_bool(self):
         r = pg.get_bool()
         self.assertIsInstance(r, bool)
         self.assertIs(r, True)
 
-    def testSetBool(self):
+    def test_set_bool(self):
         use_bool = pg.get_bool()
         try:
             pg.set_bool(False)
@@ -1007,12 +1007,12 @@ class TestConfigFunctions(unittest.TestCase):
         self.assertIsInstance(r, bool)
         self.assertIs(r, use_bool)
 
-    def testGetByteaEscaped(self):
+    def test_get_bytea_escaped(self):
         r = pg.get_bytea_escaped()
         self.assertIsInstance(r, bool)
         self.assertIs(r, False)
 
-    def testSetByteaEscaped(self):
+    def test_set_bytea_escaped(self):
         bytea_escaped = pg.get_bytea_escaped()
         try:
             pg.set_bytea_escaped(True)
@@ -1030,12 +1030,12 @@ class TestConfigFunctions(unittest.TestCase):
         self.assertIsInstance(r, bool)
         self.assertIs(r, bytea_escaped)
 
-    def testGetJsondecode(self):
+    def test_get_jsondecode(self):
         r = pg.get_jsondecode()
         self.assertTrue(callable(r))
         self.assertIs(r, json.loads)
 
-    def testSetJsondecode(self):
+    def test_set_jsondecode(self):
         jsondecode = pg.get_jsondecode()
         try:
             pg.set_jsondecode(None)
@@ -1054,7 +1054,7 @@ class TestConfigFunctions(unittest.TestCase):
 class TestModuleConstants(unittest.TestCase):
     """Test the existence of the documented module constants."""
 
-    def testVersion(self):
+    def test_version(self):
         v = pg.version
         self.assertIsInstance(v, str)
         # make sure the version conforms to PEP440
