@@ -1,27 +1,18 @@
 #!/usr/bin/python
-# -*- coding: utf-8 -*-
+
+from __future__ import annotations
 
 import gc
-import sys
 import unittest
-
-from datetime import date, time, datetime, timedelta
-from uuid import UUID as Uuid
+from datetime import date, datetime, time, timedelta, timezone
+from decimal import Decimal
+from typing import Any, ClassVar
+from uuid import UUID as Uuid  # noqa: N811
 
 import pgdb
 
-try:
-    from . import dbapi20
-except (ImportError, ValueError, SystemError):
-    # noinspection PyUnresolvedReferences
-    import dbapi20
-
-from .config import dbname, dbhost, dbport, dbuser, dbpasswd
-
-try:  # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
-    long
-except NameError:  # Python >= 3.0
-    long = int
+from . import dbapi20
+from .config import dbhost, dbname, dbpasswd, dbport, dbuser
 
 
 class PgBitString:
@@ -31,21 +22,21 @@ class PgBitString:
         self.value = value
 
     def __pg_repr__(self):
-        return "B'{0:b}'".format(self.value)
+        return f"B'{self.value:b}'"
 
 
-class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
+class TestPgDb(dbapi20.DatabaseAPI20Test):
 
     driver = pgdb
     connect_args = ()
-    connect_kw_args = {
-        'database': dbname, 'host': '%s:%d' % (dbhost or '', dbport or -1),
+    connect_kw_args: ClassVar[dict[str, Any]] = {
+        'database': dbname, 'host': f"{dbhost or ''}:{dbport or -1}",
         'user': dbuser, 'password': dbpasswd}
 
     lower_func = 'lower'  # For stored procedure test
 
     def setUp(self):
-        dbapi20.DatabaseAPI20Test.setUp(self)
+        super().setUp()
         try:
             con = self._connect()
             con.close()
@@ -59,7 +50,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             db.query('create database ' + dbname)
 
     def tearDown(self):
-        dbapi20.DatabaseAPI20Test.tearDown(self)
+        super().tearDown()
 
     def test_version(self):
         v = pgdb.version
@@ -171,8 +162,10 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         class TestCursor(pgdb.Cursor):
 
             def row_factory(self, row):
-                return {'column %s' % desc[0]: value
-                        for desc, value in zip(self.description, row)}
+                description = self.description
+                assert isinstance(description, list)
+                return {f'column {desc[0]}': value
+                        for desc, value in zip(description, row)}
 
         con = self._connect()
         con.cursor_type = TestCursor
@@ -198,7 +191,9 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         class TestCursor(pgdb.Cursor):
 
             def build_row_factory(self):
-                keys = [desc[0] for desc in self.description]
+                description = self.description
+                assert isinstance(description, list)
+                keys = [desc[0] for desc in description]
                 return lambda row: {
                     key: value for key, value in zip(keys, row)}
 
@@ -313,7 +308,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             self.assertIsInstance(d, tuple)
             self.assertEqual(len(d), 7)
             self.assertIsInstance(d.name, str)
-            self.assertEqual(d.name, 'col%d' % i)
+            self.assertEqual(d.name, f'col{i}')
             self.assertIsInstance(d.type_code, str)
             self.assertEqual(d.type_code, c[0])
             self.assertIsNone(d.display_size)
@@ -389,7 +384,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             cur = con.cursor()
             type_cache = con.type_cache
             self.assertIs(type_cache.get_typecast('int4'), int)
-            cast_int = lambda v: 'int(%s)' % v  # noqa: E731
+            cast_int = lambda v: f'int({v})'  # noqa: E731
             type_cache.set_typecast('int4', cast_int)
             query = 'select 2::int2, 4::int4, 8::int8'
             cur.execute(query)
@@ -449,7 +444,6 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         self.assertRaises(pgdb.OperationalError, cur.fetchone)
 
     def test_fetch_2_rows(self):
-        Decimal = pgdb.decimal_type()
         values = ('test', pgdb.Binary(b'\xff\x52\xb2'),
                   True, 5, 6, 5.7, Decimal('234.234234'), Decimal('75.45'),
                   pgdb.Date(2011, 7, 17), pgdb.Time(15, 47, 42),
@@ -461,7 +455,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             cur = con.cursor()
             cur.execute("set datestyle to iso")
             cur.execute(
-                "create table %s ("
+                f"create table {table} ("
                 "stringtest varchar,"
                 "binarytest bytea,"
                 "booltest bool,"
@@ -474,16 +468,16 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
                 "timetest time,"
                 "datetimetest timestamp,"
                 "intervaltest interval,"
-                "rowidtest oid)" % table)
+                "rowidtest oid)")
             cur.execute("set standard_conforming_strings to on")
             for s in ('numeric', 'monetary', 'time'):
-                cur.execute("set lc_%s to 'C'" % s)
+                cur.execute(f"set lc_{s} to 'C'")
             for _i in range(2):
                 cur.execute(
-                    "insert into %s values ("
-                    "%%s,%%s,%%s,%%s,%%s,%%s,%%s,"
-                    "'%%s'::money,%%s,%%s,%%s,%%s,%%s)" % table, values)
-            cur.execute("select * from %s" % table)
+                    f"insert into {table} values ("
+                    "%s,%s,%s,%s,%s,%s,%s,"
+                    "'%s'::money,%s,%s,%s,%s,%s)", values)
+            cur.execute(f"select * from {table}")
             rows = cur.fetchall()
             self.assertEqual(len(rows), 2)
             row0 = rows[0]
@@ -493,7 +487,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             self.assertIsInstance(row0[1], bytes)
             self.assertIsInstance(row0[2], bool)
             self.assertIsInstance(row0[3], int)
-            self.assertIsInstance(row0[4], long)
+            self.assertIsInstance(row0[4], int)
             self.assertIsInstance(row0[5], float)
             self.assertIsInstance(row0[6], Decimal)
             self.assertIsInstance(row0[7], Decimal)
@@ -510,12 +504,12 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         try:
             cur = con.cursor()
             cur.execute("set client_min_messages = warning")
-            cur.execute("create table %s (i int primary key)" % table)
-            cur.execute("insert into %s values (1)" % table)
-            cur.execute("insert into %s values (2)" % table)
+            cur.execute(f"create table {table} (i int primary key)")
+            cur.execute(f"insert into {table} values (1)")
+            cur.execute(f"insert into {table} values (2)")
             self.assertRaises(
                 pgdb.IntegrityError, cur.execute,
-                "insert into %s values (1)" % table)
+                f"insert into {table} values (1)")
         finally:
             con.close()
 
@@ -524,11 +518,11 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         con = self._connect()
         try:
             cur = con.cursor()
-            cur.execute("create table %s (i int)" % table)
-            cur.execute("insert into %s values (1)" % table)
-            cur.execute("update %s set i=2 where i=2 returning i" % table)
+            cur.execute(f"create table {table} (i int)")
+            cur.execute(f"insert into {table} values (1)")
+            cur.execute(f"update {table} set i=2 where i=2 returning i")
             self.assertEqual(cur.rowcount, 0)
-            cur.execute("update %s set i=2 where i=1 returning i" % table)
+            cur.execute(f"update {table} set i=2 where i=1 returning i")
             self.assertEqual(cur.rowcount, 1)
             cur.close()
             # keep rowcount even if cursor is closed (needed by SQLAlchemy)
@@ -542,14 +536,14 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         try:
             cur.execute("select 1/0")
         except pgdb.DatabaseError as error:
-            self.assertTrue(isinstance(error, pgdb.DataError))
+            self.assertIsInstance(error, pgdb.DataError)
             # the SQLSTATE error code for division by zero is 22012
             # noinspection PyUnresolvedReferences
             self.assertEqual(error.sqlstate, '22012')
 
     def test_float(self):
         nan, inf = float('nan'), float('inf')
-        from math import isnan, isinf
+        from math import isinf, isnan
         self.assertTrue(isnan(nan) and not isinf(nan))
         self.assertTrue(isinf(inf) and not isnan(inf))
         values = [0, 1, 0.03125, -42.53125, nan, inf, -inf,
@@ -559,10 +553,10 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         try:
             cur = con.cursor()
             cur.execute(
-                "create table %s (n smallint, floattest float)" % table)
+                f"create table {table} (n smallint, floattest float)")
             params = enumerate(values)
-            cur.executemany("insert into %s values (%%d,%%s)" % table, params)
-            cur.execute("select floattest from %s order by n" % table)
+            cur.executemany(f"insert into {table} values (%d,%s)", params)
+            cur.execute(f"select floattest from {table} order by n")
             rows = cur.fetchall()
             self.assertEqual(cur.description[0].type_code, pgdb.FLOAT)
             self.assertNotEqual(cur.description[0].type_code, pgdb.ARRAY)
@@ -578,60 +572,68 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
                 inval = -inf
             elif inval in ('nan', 'NaN'):
                 inval = nan
-            if isinf(inval):
+            if isinf(inval):  # type: ignore
                 self.assertTrue(isinf(outval))
-                if inval < 0:
-                    self.assertTrue(outval < 0)
+                if inval < 0:  # type: ignore
+                    self.assertLess(outval, 0)
                 else:
-                    self.assertTrue(outval > 0)
-            elif isnan(inval):
+                    self.assertGreater(outval, 0)
+            elif isnan(inval):  # type: ignore
                 self.assertTrue(isnan(outval))
             else:
                 self.assertEqual(inval, outval)
 
     def test_datetime(self):
         dt = datetime(2011, 7, 17, 15, 47, 42, 317509)
+        values = [dt.date(), dt.time(), dt, dt.time(), dt]
+        self.assertIsInstance(values[3], time)
+        assert isinstance(values[3], time)  # type guard
+        values[3] = values[3].replace(tzinfo=timezone.utc)
+        self.assertIsInstance(values[4], datetime)
+        assert isinstance(values[4], datetime)  # type guard
+        values[4] = values[4].replace(tzinfo=timezone.utc)
+        da = (dt.year, dt.month, dt.day)
+        ti = (dt.hour, dt.minute, dt.second, dt.microsecond)
+        tz = (timezone.utc,)
+        inputs = [
+            # input as objects
+            values,
+            # input as text
+            [v.isoformat() for v in values],  # type: ignore
+            # # input using type helpers
+            [pgdb.Date(*da), pgdb.Time(*ti),
+             pgdb.Timestamp(*(da + ti)), pgdb.Time(*(ti + tz)),
+             pgdb.Timestamp(*(da + ti + tz))]
+        ]
         table = self.table_prefix + 'booze'
-        con = self._connect()
+        con: pgdb.Connection = self._connect()
         try:
             cur = con.cursor()
             cur.execute("set timezone = UTC")
-            cur.execute("create table %s ("
+            cur.execute(f"create table {table} ("
                         "d date, t time,  ts timestamp,"
-                        "tz timetz, tsz timestamptz)" % table)
-            for n in range(3):
-                values = [dt.date(), dt.time(), dt, dt.time(), dt]
-                values[3] = values[3].replace(tzinfo=pgdb.timezone.utc)
-                values[4] = values[4].replace(tzinfo=pgdb.timezone.utc)
-                if n == 0:  # input as objects
-                    params = values
-                if n == 1:  # input as text
-                    params = [v.isoformat() for v in values]  # as text
-                elif n == 2:  # input using type helpers
-                    d = (dt.year, dt.month, dt.day)
-                    t = (dt.hour, dt.minute, dt.second, dt.microsecond)
-                    z = (pgdb.timezone.utc,)
-                    params = [pgdb.Date(*d), pgdb.Time(*t),
-                              pgdb.Timestamp(*(d + t)), pgdb.Time(*(t + z)),
-                              pgdb.Timestamp(*(d + t + z))]
+                        "tz timetz, tsz timestamptz)")
+            for params in inputs:
                 for datestyle in ('iso', 'postgres, mdy', 'postgres, dmy',
                                   'sql, mdy', 'sql, dmy', 'german'):
-                    cur.execute("set datestyle to %s" % datestyle)
-                    if n != 1:
-                        # noinspection PyUnboundLocalVariable
+                    cur.execute(f"set datestyle to {datestyle}")
+                    if not isinstance(params[0], str):
                         cur.execute("select %s,%s,%s,%s,%s", params)
                         row = cur.fetchone()
                         self.assertEqual(row, tuple(values))
                     cur.execute(
-                        "insert into %s"
-                        " values (%%s,%%s,%%s,%%s,%%s)" % table, params)
-                    cur.execute("select * from %s" % table)
+                        f"insert into {table}"
+                        " values (%s,%s,%s,%s,%s)", params)
+                    cur.execute(f"select * from {table}")
                     d = cur.description
+                    self.assertIsInstance(d, list)
+                    assert d is not None  # type guard
                     for i in range(5):
-                        self.assertEqual(d[i].type_code, pgdb.DATETIME)
-                        self.assertNotEqual(d[i].type_code, pgdb.STRING)
-                        self.assertNotEqual(d[i].type_code, pgdb.ARRAY)
-                        self.assertNotEqual(d[i].type_code, pgdb.RECORD)
+                        tc = d[i].type_code
+                        self.assertEqual(tc, pgdb.DATETIME)
+                        self.assertNotEqual(tc, pgdb.STRING)
+                        self.assertNotEqual(tc, pgdb.ARRAY)
+                        self.assertNotEqual(tc, pgdb.RECORD)
                     self.assertEqual(d[0].type_code, pgdb.DATE)
                     self.assertEqual(d[1].type_code, pgdb.TIME)
                     self.assertEqual(d[2].type_code, pgdb.TIMESTAMP)
@@ -639,33 +641,32 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
                     self.assertEqual(d[4].type_code, pgdb.TIMESTAMP)
                     row = cur.fetchone()
                     self.assertEqual(row, tuple(values))
-                    cur.execute("truncate table %s" % table)
+                    cur.execute(f"truncate table {table}")
         finally:
             con.close()
 
     def test_interval(self):
         td = datetime(2011, 7, 17, 15, 47, 42, 317509) - datetime(1970, 1, 1)
+        inputs = [
+            # input as objects
+            td,
+            # input as text
+            f'{td.days} days {td.seconds} seconds'
+            f' {td.microseconds} microseconds',
+            # input using type helpers
+            pgdb.Interval(td.days, 0, 0, td.seconds, td.microseconds)]
         table = self.table_prefix + 'booze'
         con = self._connect()
         try:
             cur = con.cursor()
-            cur.execute("create table %s (i interval)" % table)
-            for n in range(3):
-                if n == 0:  # input as objects
-                    param = td
-                if n == 1:  # input as text
-                    param = '%d days %d seconds %d microseconds ' % (
-                        td.days, td.seconds, td.microseconds)
-                elif n == 2:  # input using type helpers
-                    param = pgdb.Interval(
-                        td.days, 0, 0, td.seconds, td.microseconds)
+            cur.execute(f"create table {table} (i interval)")
+            for param in inputs:
                 for intervalstyle in ('sql_standard ', 'postgres',
                                       'postgres_verbose', 'iso_8601'):
-                    cur.execute("set intervalstyle to %s" % intervalstyle)
+                    cur.execute(f"set intervalstyle to {intervalstyle}")
                     # noinspection PyUnboundLocalVariable
-                    cur.execute("insert into %s"
-                                " values (%%s)" % table, [param])
-                    cur.execute("select * from %s" % table)
+                    cur.execute(f"insert into {table} values (%s)", [param])
+                    cur.execute(f"select * from {table}")
                     tc = cur.description[0].type_code
                     self.assertEqual(tc, pgdb.DATETIME)
                     self.assertNotEqual(tc, pgdb.STRING)
@@ -674,7 +675,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
                     self.assertEqual(tc, pgdb.INTERVAL)
                     row = cur.fetchone()
                     self.assertEqual(row, (td,))
-                    cur.execute("truncate table %s" % table)
+                    cur.execute(f"truncate table {table}")
         finally:
             con.close()
 
@@ -718,7 +719,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         self.assertEqual(result, d)
 
     def test_insert_array(self):
-        values = [
+        values: list[tuple[Any, Any]] = [
             (None, None), ([], []), ([None], [[None], ['null']]),
             ([1, 2, 3], [['a', 'b'], ['c', 'd']]),
             ([20000, 25000, 25000, 30000],
@@ -728,15 +729,15 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         con = self._connect()
         try:
             cur = con.cursor()
-            cur.execute("create table %s"
-                        " (n smallint, i int[], t text[][])" % table)
+            cur.execute(
+                f"create table {table} (n smallint, i int[], t text[][])")
             params = [(n, v[0], v[1]) for n, v in enumerate(values)]
             # Note that we must explicit casts because we are inserting
             # empty arrays.  Otherwise this is not necessary.
             cur.executemany(
-                "insert into %s values"
-                " (%%d,%%s::int[],%%s::text[][])" % table, params)
-            cur.execute("select i, t from %s order by n" % table)
+                f"insert into {table} values"
+                " (%d,%s::int[],%s::text[][])", params)
+            cur.execute(f"select i, t from {table} order by n")
             d = cur.description
             self.assertEqual(d[0].type_code, pgdb.ARRAY)
             self.assertNotEqual(d[0].type_code, pgdb.RECORD)
@@ -762,7 +763,7 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         self.assertEqual(row, values)
 
     def test_unicode_list_and_tuple(self):
-        value = (u'Käse', u'Würstchen')
+        value = ('Käse', 'Würstchen')
         con = self._connect()
         try:
             cur = con.cursor()
@@ -787,11 +788,11 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         con = self._connect()
         cur = con.cursor()
         try:
-            cur.execute("create type %s as (name varchar, age int)" % record)
-            cur.execute("create table %s (n smallint, r %s)" % (table, record))
+            cur.execute(f"create type {record} as (name varchar, age int)")
+            cur.execute(f"create table {table} (n smallint, r {record})")
             params = enumerate(values)
-            cur.executemany("insert into %s values (%%d,%%s)" % table, params)
-            cur.execute("select r from %s order by n" % table)
+            cur.executemany(f"insert into {table} values (%d,%s)", params)
+            cur.execute(f"select r from {table} order by n")
             type_code = cur.description[0].type_code
             self.assertEqual(type_code, record)
             self.assertEqual(type_code, pgdb.RECORD)
@@ -803,8 +804,8 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             self.assertEqual(con.type_cache[columns[1].type], 'int4')
             rows = cur.fetchall()
         finally:
-            cur.execute('drop table %s' % table)
-            cur.execute('drop type %s' % record)
+            cur.execute(f'drop table {table}')
+            cur.execute(f'drop type {record}')
             con.close()
         self.assertEqual(len(rows), len(values))
         rows = [row[0] for row in rows]
@@ -832,16 +833,16 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
 
     def test_custom_type(self):
         values = [3, 5, 65]
-        values = list(map(PgBitString, values))
+        values = list(map(PgBitString, values))  # type: ignore
         table = self.table_prefix + 'booze'
         con = self._connect()
         try:
             cur = con.cursor()
-            params = enumerate(values)  # params have __pg_repr__ method
+            seq_params = enumerate(values)  # params have __pg_repr__ method
             cur.execute(
-                'create table "%s" (n smallint, b bit varying(7))' % table)
-            cur.executemany("insert into %s values (%%s,%%s)" % table, params)
-            cur.execute("select * from %s" % table)
+                f'create table "{table}" (n smallint, b bit varying(7))')
+            cur.executemany(f"insert into {table} values (%s,%s)", seq_params)
+            cur.execute(f"select * from {table}")
             rows = cur.fetchall()
         finally:
             con.close()
@@ -852,49 +853,58 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             params = (1, object())  # an object that cannot be handled
             self.assertRaises(
                 pgdb.InterfaceError, cur.execute,
-                "insert into %s values (%%s,%%s)" % table, params)
+                f"insert into {table} values (%s,%s)", params)
         finally:
             con.close()
 
     def test_set_decimal_type(self):
-        decimal_type = pgdb.decimal_type()
-        self.assertTrue(decimal_type is not None and callable(decimal_type))
+        from pgdb.cast import decimal_type
+        self.assertIs(decimal_type(), Decimal)
         con = self._connect()
         try:
             cur = con.cursor()
             # change decimal type globally to int
-            int_type = lambda v: int(float(v))  # noqa: E731
-            self.assertTrue(pgdb.decimal_type(int_type) is int_type)
+
+            class CustomDecimal(str):
+
+                def __init__(self, value: Any) -> None:
+                    self.value = value
+
+                def __str__(self) -> str:
+                    return str(self.value).replace('.', ',')
+
+            self.assertIs(decimal_type(CustomDecimal), CustomDecimal)
             cur.execute('select 4.25')
             self.assertEqual(cur.description[0].type_code, pgdb.NUMBER)
             value = cur.fetchone()[0]
-            self.assertTrue(isinstance(value, int))
-            self.assertEqual(value, 4)
+            self.assertIsInstance(value, CustomDecimal)
+            self.assertEqual(str(value), '4,25')
             # change decimal type again to float
-            self.assertTrue(pgdb.decimal_type(float) is float)
+            self.assertIs(decimal_type(float), float)
             cur.execute('select 4.25')
             self.assertEqual(cur.description[0].type_code, pgdb.NUMBER)
             value = cur.fetchone()[0]
             # the connection still uses the old setting
-            self.assertTrue(isinstance(value, int))
+            self.assertIsInstance(value, str)
+            self.assertEqual(str(value), '4,25')
             # bust the cache for type functions for the connection
             con.type_cache.reset_typecast()
             cur.execute('select 4.25')
             self.assertEqual(cur.description[0].type_code, pgdb.NUMBER)
             value = cur.fetchone()[0]
             # now the connection uses the new setting
-            self.assertTrue(isinstance(value, float))
+            self.assertIsInstance(value, float)
             self.assertEqual(value, 4.25)
         finally:
             con.close()
-            pgdb.decimal_type(decimal_type)
-        self.assertTrue(pgdb.decimal_type() is decimal_type)
+            decimal_type(Decimal)
+        self.assertIs(decimal_type(), Decimal)
 
     def test_global_typecast(self):
         try:
             query = 'select 2::int2, 4::int4, 8::int8'
             self.assertIs(pgdb.get_typecast('int4'), int)
-            cast_int = lambda v: 'int(%s)' % v  # noqa: E731
+            cast_int = lambda v: f'int({v})'  # noqa: E731
             pgdb.set_typecast('int4', cast_int)
             con = self._connect()
             try:
@@ -981,28 +991,26 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
 
     def test_unicode_with_utf8(self):
         table = self.table_prefix + 'booze'
-        s = u"He wes Leovenaðes sone — liðe him be Drihten"
+        s = "He wes Leovenaðes sone — liðe him be Drihten"
         con = self._connect()
         cur = con.cursor()
         try:
-            cur.execute("create table %s (t text)" % table)
+            cur.execute(f"create table {table} (t text)")
             try:
                 cur.execute("set client_encoding=utf8")
-                cur.execute(u"select '%s'" % s)
+                cur.execute(f"select '{s}'")
             except Exception:
                 self.skipTest("database does not support utf8")
             output1 = cur.fetchone()[0]
-            cur.execute("insert into %s values (%%s)" % table, (s,))
-            cur.execute("select * from %s" % table)
+            cur.execute(f"insert into {table} values (%s)", (s,))
+            cur.execute(f"select * from {table}")
             output2 = cur.fetchone()[0]
-            cur.execute("select t = '%s' from %s" % (s, table))
+            cur.execute(f"select t = '{s}' from {table}")
             output3 = cur.fetchone()[0]
-            cur.execute("select t = %%s from %s" % table, (s,))
+            cur.execute(f"select t = %s from {table}", (s,))
             output4 = cur.fetchone()[0]
         finally:
             con.close()
-        if str is bytes:  # Python < 3.0
-            s = s.encode('utf8')
         self.assertIsInstance(output1, str)
         self.assertEqual(output1, s)
         self.assertIsInstance(output2, str)
@@ -1014,28 +1022,26 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
 
     def test_unicode_with_latin1(self):
         table = self.table_prefix + 'booze'
-        s = u"Ehrt den König seine Würde, ehret uns der Hände Fleiß."
+        s = "Ehrt den König seine Würde, ehret uns der Hände Fleiß."
         con = self._connect()
         try:
             cur = con.cursor()
-            cur.execute("create table %s (t text)" % table)
+            cur.execute(f"create table {table} (t text)")
             try:
                 cur.execute("set client_encoding=latin1")
-                cur.execute(u"select '%s'" % s)
+                cur.execute(f"select '{s}'")
             except Exception:
                 self.skipTest("database does not support latin1")
             output1 = cur.fetchone()[0]
-            cur.execute("insert into %s values (%%s)" % table, (s,))
-            cur.execute("select * from %s" % table)
+            cur.execute(f"insert into {table} values (%s)", (s,))
+            cur.execute(f"select * from {table}")
             output2 = cur.fetchone()[0]
-            cur.execute("select t = '%s' from %s" % (s, table))
+            cur.execute(f"select t = '{s}' from {table}")
             output3 = cur.fetchone()[0]
-            cur.execute("select t = %%s from %s" % table, (s,))
+            cur.execute(f"select t = %s from {table}", (s,))
             output4 = cur.fetchone()[0]
         finally:
             con.close()
-        if str is bytes:  # Python < 3.0
-            s = s.encode('latin1')
         self.assertIsInstance(output1, str)
         self.assertEqual(output1, s)
         self.assertIsInstance(output2, str)
@@ -1051,11 +1057,10 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         con = self._connect()
         try:
             cur = con.cursor()
-            cur.execute(
-                "create table %s (n smallint, booltest bool)" % table)
+            cur.execute(f"create table {table} (n smallint, booltest bool)")
             params = enumerate(values)
-            cur.executemany("insert into %s values (%%s,%%s)" % table, params)
-            cur.execute("select booltest from %s order by n" % table)
+            cur.executemany(f"insert into {table} values (%s,%s)", params)
+            cur.execute(f"select booltest from {table} order by n")
             rows = cur.fetchall()
             self.assertEqual(cur.description[0].type_code, pgdb.BOOL)
         finally:
@@ -1084,12 +1089,12 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         try:
             cur = con.cursor()
             try:
-                cur.execute("create table %s (jsontest json)" % table)
+                cur.execute(f"create table {table} (jsontest json)")
             except pgdb.ProgrammingError:
                 self.skipTest('database does not support json')
             params = (pgdb.Json(inval),)
-            cur.execute("insert into %s values (%%s)" % table, params)
-            cur.execute("select jsontest from %s" % table)
+            cur.execute(f"insert into {table} values (%s)", params)
+            cur.execute(f"select jsontest from {table}")
             outval = cur.fetchone()[0]
             self.assertEqual(cur.description[0].type_code, pgdb.JSON)
         finally:
@@ -1104,12 +1109,12 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         try:
             cur = con.cursor()
             try:
-                cur.execute("create table %s (jsonbtest jsonb)" % table)
+                cur.execute(f"create table {table} (jsonbtest jsonb)")
             except pgdb.ProgrammingError:
                 self.skipTest('database does not support jsonb')
             params = (pgdb.Json(inval),)
-            cur.execute("insert into %s values (%%s)" % table, params)
-            cur.execute("select jsonbtest from %s" % table)
+            cur.execute(f"insert into {table} values (%s)", params)
+            cur.execute(f"select jsonbtest from {table}")
             outval = cur.fetchone()[0]
             self.assertEqual(cur.description[0].type_code, pgdb.JSON)
         finally:
@@ -1146,8 +1151,8 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             for n in (1, 3, 5, 7, 10, 100, 1000):
                 cur = con.cursor()
                 try:
-                    cur.execute('select n, n::text as s, n %% 2 = 1 as b'
-                                ' from generate_series(1, %d) as s(n)' % n)
+                    cur.execute('select n, n::text as s, n % 2 = 1 as b'
+                                f' from generate_series(1, {n}) as s(n)')
                     res = cur.fetchall()
                     self.assertEqual(len(res), n, res)
                     self.assertEqual(len(res[0]), 3)
@@ -1188,10 +1193,10 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         finally:
             con.close()
 
-    def help_nextset_setUp(self, _cur):
+    def help_nextset_setup(self, _cur):
         pass  # helper not needed
 
-    def help_nextset_tearDown(self, _cur):
+    def help_nextset_teardown(self, _cur):
         pass  # helper not needed
 
     def test_nextset(self):
@@ -1219,17 +1224,17 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         table = self.table_prefix + 'booze'
         con1 = self._connect()
         cur1 = con1.cursor()
-        self.executeDDL1(cur1)
+        self.execute_ddl1(cur1)
         con1.commit()
         con2 = self._connect()
         cur2 = con2.cursor()
-        cur2.execute("select name from %s" % table)
+        cur2.execute(f"select name from {table}")
         self.assertIsNone(cur2.fetchone())
-        cur1.execute("insert into %s values('Schlafly')" % table)
-        cur2.execute("select name from %s" % table)
+        cur1.execute(f"insert into {table} values('Schlafly')")
+        cur2.execute(f"select name from {table}")
         self.assertIsNone(cur2.fetchone())
         con1.commit()
-        cur2.execute("select name from %s" % table)
+        cur2.execute(f"select name from {table}")
         self.assertEqual(cur2.fetchone(), ('Schlafly',))
         con2.close()
         con1.close()
@@ -1239,13 +1244,13 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         con1 = self._connect()
         con1.autocommit = True
         cur1 = con1.cursor()
-        self.executeDDL1(cur1)
+        self.execute_ddl1(cur1)
         con2 = self._connect()
         cur2 = con2.cursor()
-        cur2.execute("select name from %s" % table)
+        cur2.execute(f"select name from {table}")
         self.assertIsNone(cur2.fetchone())
-        cur1.execute("insert into %s values('Shmaltz Pastrami')" % table)
-        cur2.execute("select name from %s" % table)
+        cur1.execute(f"insert into {table} values('Shmaltz Pastrami')")
+        cur2.execute(f"select name from {table}")
         self.assertEqual(cur2.fetchone(), ('Shmaltz Pastrami',))
         con2.close()
         con1.close()
@@ -1258,32 +1263,32 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
             try:
                 cur = con.cursor()
                 if autocommit:
-                    cur.execute("truncate table %s" % table)
+                    cur.execute(f"truncate table {table}")
                 else:
                     cur.execute(
-                        "create table %s (n smallint check(n!=4))" % table)
+                        f"create table {table} (n smallint check(n!=4))")
                 with con:
-                    cur.execute("insert into %s values (1)" % table)
-                    cur.execute("insert into %s values (2)" % table)
+                    cur.execute(f"insert into {table} values (1)")
+                    cur.execute(f"insert into {table} values (2)")
                 try:
                     with con:
-                        cur.execute("insert into %s values (3)" % table)
-                        cur.execute("insert into %s values (4)" % table)
+                        cur.execute(f"insert into {table} values (3)")
+                        cur.execute(f"insert into {table} values (4)")
                 except con.IntegrityError as error:
-                    self.assertTrue('check' in str(error).lower())
+                    self.assertIn('check', str(error).lower())
                 with con:
-                    cur.execute("insert into %s values (5)" % table)
-                    cur.execute("insert into %s values (6)" % table)
+                    cur.execute(f"insert into {table} values (5)")
+                    cur.execute(f"insert into {table} values (6)")
                 try:
                     with con:
-                        cur.execute("insert into %s values (7)" % table)
-                        cur.execute("insert into %s values (8)" % table)
+                        cur.execute(f"insert into {table} values (7)")
+                        cur.execute(f"insert into {table} values (8)")
                         raise ValueError('transaction should rollback')
                 except ValueError as error:
                     self.assertEqual(str(error), 'transaction should rollback')
                 with con:
-                    cur.execute("insert into %s values (9)" % table)
-                cur.execute("select * from %s order by 1" % table)
+                    cur.execute(f"insert into {table} values (9)")
+                cur.execute(f"select * from {table} order by 1")
                 rows = cur.fetchall()
                 rows = [row[0] for row in rows]
             finally:
@@ -1323,11 +1328,11 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         self.assertEqual('int8', pgdb.INTEGER)
         self.assertNotEqual('int4', pgdb.LONG)
         self.assertEqual('int8', pgdb.LONG)
-        self.assertTrue('char' in pgdb.STRING)
-        self.assertTrue(pgdb.NUMERIC <= pgdb.NUMBER)
-        self.assertTrue(pgdb.NUMBER >= pgdb.INTEGER)
-        self.assertTrue(pgdb.TIME <= pgdb.DATETIME)
-        self.assertTrue(pgdb.DATETIME >= pgdb.DATE)
+        self.assertIn('char', pgdb.STRING)
+        self.assertLess(pgdb.NUMERIC, pgdb.NUMBER)
+        self.assertGreaterEqual(pgdb.NUMBER, pgdb.INTEGER)
+        self.assertLessEqual(pgdb.TIME, pgdb.DATETIME)
+        self.assertGreaterEqual(pgdb.DATETIME, pgdb.DATE)
         self.assertEqual(pgdb.ARRAY, pgdb.ARRAY)
         self.assertNotEqual(pgdb.ARRAY, pgdb.STRING)
         self.assertEqual('_char', pgdb.ARRAY)
@@ -1342,22 +1347,19 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         data = ('hello', 'world')
         con = self._connect()
         cur = con.cursor()
-        cur.build_row_factory = lambda: tuple  # noqa: E731
+        cur.build_row_factory = lambda: tuple
         cur.execute("select %s, %s", data)
         row = cur.fetchone()
         self.assertEqual(row, data)
 
-    def test_set_row_factory_size(self):
-        try:
-            from functools import lru_cache
-        except ImportError:  # Python < 3.2
-            lru_cache = None
+    def test_change_row_factory_cache_size(self):
+        from pg import RowCache
         queries = ['select 1 as a, 2 as b, 3 as c', 'select 123 as abc']
         con = self._connect()
         cur = con.cursor()
         for maxsize in (None, 0, 1, 2, 3, 10, 1024):
-            pgdb.set_row_factory_size(maxsize)
-            for i in range(3):
+            RowCache.change_size(maxsize)
+            for _i in range(3):
                 for q in queries:
                     cur.execute(q)
                     r = cur.fetchone()
@@ -1367,16 +1369,15 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
                     else:
                         self.assertEqual(r, (1, 2, 3))
                         self.assertEqual(r._fields, ('a', 'b', 'c'))
-            if lru_cache:
-                info = pgdb._row_factory.cache_info()
-                self.assertEqual(info.maxsize, maxsize)
-                self.assertEqual(info.hits + info.misses, 6)
-                self.assertEqual(
-                    info.hits, 0 if maxsize is not None and maxsize < 2 else 4)
+            info = RowCache.row_factory.cache_info()
+            self.assertEqual(info.maxsize, maxsize)
+            self.assertEqual(info.hits + info.misses, 6)
+            self.assertEqual(info.hits,
+                             0 if maxsize is not None and maxsize < 2 else 4)
 
     def test_memory_leaks(self):
-        ids = set()
-        objs = []
+        ids: set = set()
+        objs: list = []
         add_ids = ids.update
         gc.collect()
         objs[:] = gc.get_objects()
@@ -1385,9 +1386,6 @@ class test_PyGreSQL(dbapi20.DatabaseAPI20Test):
         gc.collect()
         objs[:] = gc.get_objects()
         objs[:] = [obj for obj in objs if id(obj) not in ids]
-        if objs and sys.version_info[:3] in ((3, 5, 0), (3, 5, 1)):
-            # workaround for Python issue 26811
-            objs[:] = [obj for obj in objs if repr(obj) != '(<NULL>,)']
         self.assertEqual(len(objs), 0)
 
     def test_cve_2018_1058(self):
